// global.h

#define MY_PLATFORM_WIN32 1
#define MY_PLATFORM_LINUX 2

// ====================================================
// Change here to compile for another platform

//#define PLATFORM MY_PLATFORM_LINUX
#define PLATFORM MY_PLATFORM_WIN32

// ====================================================

#if PLATFORM == MY_PLATFORM_WIN32
	#include <Windows.h>
	#include <time.h>
	#include "./irrlicht-1.0/include/IMeshCache.h"
	#include "./irrlicht-1.0/include/irrlicht.h"
	#pragma comment(lib, "./irrlicht-1.0/lib/Irrlicht.lib")
#else
	#include <stdio.h>
	#include <stdlib.h>
	#include <string.h>
	#include <time.h>
	#include "./irrlicht-1.0/include/IMeshCache.h"
	#include "./irrlicht-1.0/include/irrlicht.h"
#endif

// ====================================================
// My definitions. 

#define SAFERELEASE(p)      if (p) {(p)->Release(); (p) = NULL;}
#define SAFEDELETE(p)       if (p) {delete(p);      (p) = NULL;}
#define SAFEDELETE_ARRAY(p) if (p) {delete[](p);    (p) = NULL;}
#define SAFEREMOVE(p)       if (p) {(p)->remove();  (p) = NULL;}
#define SAFEDROP(p)         if (p) {(p)->drop();    (p) = NULL;}

#define MIN(a, b) ((a) < (b) ? (a) : (b))						
#define MAX(a, b) ((a) > (b) ? (a) : (b))

#define PI 3.1415926535f

#define DEGTORAD(x) ((x) * PI / 180.0f)
#define RADTODEG(x) ((x) * 180.0f / PI)

// ====================================================
// Global includes

#include "CGUIexScrollWindow.h"
#include "input.h"
#include "camera.h"
#include "map.h"
#include "gui.h"
#include "debug.h"

// ====================================================
// The global data structures 

typedef struct 
{
	irr::IrrlichtDevice*		  pIrrlichtDevice;
	irr::video::IVideoDriver*	  pVideoDriver;
	irr::scene::ISceneManager*	  pSceneManager;
	irr::gui::IGUIEnvironment*	  pGUIEnv;
	irr::ILogger*				  pLogger;

	irr::core::dimension2d<irr::s32> ScreenXY;
	irr::u32 uColorDepth;

	CInput   Input;
	CMap     Map;
	CCamera  Camera;
	CGui     Gui;
	CDebug   Debug;

} GLOBALDATA;

// ====================================================
// Debug Helpers

// Inline function for easy debug output (string)
inline void DB(char* pInfo, irr::ELOG_LEVEL lvl = irr::ELL_INFORMATION) 
{
    extern GLOBALDATA gl_Data;
	char buffer[2048];
	sprintf(buffer,"IrrChess: %s",pInfo);
    gl_Data.pLogger->log(buffer, lvl);
}

// Inline function for easy debug output (string+float)
inline void DB(char* pInfo, float fVal, irr::ELOG_LEVEL lvl = irr::ELL_INFORMATION) 
{
    extern GLOBALDATA gl_Data;
	char buffer[2048];
    sprintf(buffer,"IrrChess: %s %f",pInfo,fVal);
    gl_Data.pLogger->log(buffer,lvl);
}

// Inline function for easy debug output (string+irr::s32)
inline void DB(char* pInfo, irr::s32 iVal, irr::ELOG_LEVEL lvl = irr::ELL_INFORMATION) 
{
    extern GLOBALDATA gl_Data;
	char buffer[2048];
    sprintf(buffer,"IrrChess: %s %i",pInfo,iVal);
    gl_Data.pLogger->log(buffer, lvl);
}

// ====================================================
// Global functions available in main.cpp

bool gl_Init(void);
void gl_Loop(void);
void gl_Exit(void);

